import base64
import time
import requests
from pathlib import Path
import shutil
import yaml
# 2023-05-13 import oschmod
import re
import os
import stat
from urllib.parse import urlencode, quote_plus
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_datetime
from qualys_etl.etld_lib import etld_lib_oschmod as oschmod
from qualys_etl.etld_lib import etld_lib_qualys_platform_identification
import qualys_etl
from qualys_etl.etld_lib import etld_lib_authentication_objects

global cred_dir
global cookie_file
global bearer_file
global cred_file
global use_cookie
global login_failed
global login_gateway_failed
global http_return_code
global platform_url
global injected_stdin_cred
global qualys_authentication_obj


# https://www.qualys.com/platform-identification/
# platform_url = {
#     'qualysapi.qualys.com': 'gateway.qg1.apps.qualys.com',
#     'qualysapi.qg2.apps.qualys.com': 'gateway.qg2.apps.qualys.com',
#     'qualysapi.qg3.apps.qualys.com': 'gateway.qg3.apps.qualys.com',
#     'qualysapi.qg4.apps.qualys.com': 'gateway.qg4.apps.qualys.com',
#     'qualysapi.qualys.eu': 'gateway.qg1.apps.qualys.eu',
#     'qualysapi.qg2.apps.qualys.eu': 'gateway.qg2.apps.qualys.eu',
#     'qualysapi.qg3.apps.qualys.it': 'gateway.qg3.apps.qualys.it',
#     'qualysapi.qg1.apps.qualys.in': 'gateway.qg1.apps.qualys.in',
#     'qualysapi.qg1.apps.qualys.ca': 'gateway.qg1.apps.qualys.ca',
#     'qualysapi.qg1.apps.qualys.ae': 'gateway.qg1.apps.qualys.ae',
#     'qualysapi.qg1.apps.qualys.co.uk': 'gateway.qg1.apps.qualys.co.uk',
#     'qualysapi.qg1.apps.qualys.com.au': 'gateway.qg1.apps.qualys.com.au',
#     'qualysapi.qg1.apps.qualysksa.com': 'gateway.qg1.apps.qualysksa.com',
# }

def get_qualys_headers(request=None):
    # 'X-Powered-By': 'Qualys:USPOD1:a6df6808-8c45-eb8c-e040-10ac13041e17:9e42af6e-c5a2-4d9e-825c-449440445cc8'
    # 'X-RateLimit-Limit': '2000'
    # 'X-RateLimit-Window-Sec': '3600'
    # 'X-Concurrency-Limit-Limit': '10'
    # 'X-Concurrency-Limit-Running': '0'
    # 'X-RateLimit-ToWait-Sec': '0'
    # 'X-RateLimit-Remaining': '1999'
    # 'Keep-Alive': 'timeout=300, max=250'
    # 'Connection': 'Keep-Alive'
    # 'Transfer-Encoding': 'chunked'
    # 'Content-Type': 'application/xml'
    if request is None:
        pass
    else:
        request_url = request.url
        url_fqdn = re.sub("(https://)([0-9a-zA-Z\.\_\-]+)(/.*$)", "\g<2>", request_url)
        url_end_point = re.sub("(https://[0-9a-zA-Z\.\_\-]+)/", "", request_url)
        x_ratelimit_limit = request.headers['X-RateLimit-Limit']
        x_ratelimit_window_sec = request.headers['X-RateLimit-Window-Sec']
        x_ratelimit_towait_sec = request.headers['X-RateLimit-ToWait-Sec']
        x_ratelimit_remaining = request.headers['X-RateLimit-Remaining']
        x_concurrency_limit_limit = request.headers['X-Concurrency-Limit-Limit']
        x_concurrency_limit_running = request.headers['X-Concurrency-Limit-Running']
        headers = {'url': request_url,
                   'api_fqdn_server': url_fqdn,
                   'api_end_point': url_end_point,
                   'x_ratelimit_limit': x_ratelimit_limit,
                   'x_ratelimit_window_sec': x_ratelimit_window_sec,
                   'x_ratelimit_towait_sec': x_ratelimit_towait_sec,
                   'x_ratelimit_remaining': x_ratelimit_remaining,
                   'x_concurrency_limit_limit': x_concurrency_limit_limit,
                   'x_concurrency_limit_running': x_concurrency_limit_running}
        return headers


def update_cred(new_cred):
    cred_example_file_path = Path(etld_lib_functions.qetl_code_dir, "qualys_etl", "etld_templates", ".etld_cred.yaml")
    # Get Current .etld_cred.yaml file
    with open(cred_file, 'r', encoding='utf-8') as cred_yaml_file:
        current_cred = yaml.safe_load(cred_yaml_file)
    # Get Template
    with open(str(cred_example_file_path), "r", encoding='utf-8') as cred_template_file:
        cred_template_string = cred_template_file.read()
    # Update Template # username: initialuser  password: initialpassword  api_fqdn_server: qualysapi.qualys.com
    if current_cred == new_cred:
        pass
    else:
        new_username = f"username: '{new_cred.get('username')}'"
        new_password = f"password: '{new_cred.get('password')}'"
        new_api_fqdn_server = f"api_fqdn_server: '{new_cred.get('api_fqdn_server')}'"
        # Gateway
        if new_api_fqdn_server in platform_url.keys():
            new_gateway_fqdn_server = f"gateway_fqdn_server: '{platform_url.get(new_cred.get('api_fqdn_server'))}'"
        elif 'gateway_fqdn_server' in new_cred.keys():
            new_gateway_fqdn_server = f"gateway_fqdn_server: '{new_cred.get('gateway_fqdn_server')}'"
        else:
            new_gateway_fqdn_server = "gateway.qg1.apps.qualys.com"

        local_date = etld_lib_datetime.get_local_date()
        cred_template_string = re.sub('\$DATE', local_date, cred_template_string)
        cred_template_string = re.sub('username: initialuser', new_username, cred_template_string)
        #cred_template_string = re.sub('password: initialpassword', new_password, cred_template_string)
        cred_template_string = cred_template_string.replace('password: initialpassword', new_password)
        cred_template_string = re.sub('api_fqdn_server: qualysapi.qualys.com', new_api_fqdn_server,
                                      cred_template_string)
        cred_template_string = re.sub('gateway_fqdn_server: gateway.qg1.apps.qualys.com', new_gateway_fqdn_server,
                                      cred_template_string)
        with open(str(cred_file), 'w', encoding='utf-8') as cred_file_to_update:
            cred_file_to_update.write(cred_template_string)
        oschmod.set_mode(str(cred_file), "u+rw,u-x,go-rwx")


def get_authorization(username, password):
    authorization = 'Basic ' + \
                    base64.b64encode(f"{username}:{password}".encode('utf-8')).decode('utf-8')
    return authorization


def get_env_cred() -> dict:
    username = ""
    password = ""
    api_fqdn_server = ""
    gateway_fqdn_server = ""
    authorization = ""
    bearer = ""

    k = os.environ.keys()
    if 'q_username' in k:
        username = os.environ.get('q_username')
    if 'q_password' in k:
        password = os.environ.get('q_password')
    if 'q_api_fqdn_server' in k:
        api_fqdn_server = os.environ.get('q_api_fqdn_server')
    if 'q_gateway_fqdn_server' in k:
        gateway_fqdn_server = os.environ.get('q_gateway_fqdn_server')
    if 'q_bearer' in k:
        bearer = os.environ.get('q_bearer')

    if username != "" and password != "" and api_fqdn_server != "":
        authorization = get_authorization(username, password)
        etld_lib_functions.logger.info(f"Not using .etld_cred.yaml. Found env credentials, "
                                       f"username: {username}, api_fqdn_server:  {api_fqdn_server}")
        if gateway_fqdn_server == "":
            if api_fqdn_server in platform_url.keys():
                gateway_fqdn_server = f"{platform_url.get(api_fqdn_server)}"

    return {'api_fqdn_server': api_fqdn_server,
            'gateway_fqdn_server': gateway_fqdn_server,
            'authorization': authorization,
            'username': username,
            'password': password,
            'bearer': bearer}


def get_filesystem_cred():
    try:
        with open(cred_file, 'r', encoding='utf-8') as cred_yaml_file:
            cred = yaml.safe_load(cred_yaml_file)
            api_fqdn_server = cred.get('api_fqdn_server')
            authorization = 'Basic ' + \
                            base64.b64encode(
                                f"{cred.get('username')}:{cred.get('password')}".encode('utf-8')).decode('utf-8')
            authorization_base64 = authorization.replace('Basic ', "")
            authorization_decoded_bytearray = base64.b64decode(authorization_base64)
            authorization_decoded_string = str(authorization_decoded_bytearray, 'UTF-8')
            username, password = str(authorization_decoded_string).split(':', 1)
            cred_file_mode = stat.filemode(os.stat(cred_file).st_mode)
            etld_lib_functions.logger.info(f"Found Credentials, Ensure perms are correct for your company. "
                                           f"username: {username}, api_fqdn_server:  {api_fqdn_server}, "
                                           f"permissions: {cred_file_mode} for credentials file: {cred_file}")
            if 'gateway_fqdn_server' in cred.keys():
                gateway_fqdn_server = cred.get('gateway_fqdn_server')
            elif cred.get('api_fqdn_server') in platform_url.keys():
                gateway_fqdn_server = f"{platform_url.get(cred.get('api_fqdn_server'))}"
            else:
                gateway_fqdn_server = None

            return {'api_fqdn_server': api_fqdn_server,
                    'gateway_fqdn_server': gateway_fqdn_server,
                    'authorization': authorization,
                    'username': username,
                    'password': password}

    except Exception as e:
        etld_lib_functions.logger.error(f"Please add your subscription credentials to the:  {cred_file}")
        etld_lib_functions.logger.error(
            f"   ** Warning: Ensure Credential File permissions are correct for your company.")
        etld_lib_functions.logger.error(f"   ** Warning: Credentials File: {cred_file}")
        cred_file_mode = stat.filemode(os.stat(cred_file).st_mode)
        etld_lib_functions.logger.error(f"   ** Permissions are: {cred_file_mode} for {cred_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)


def get_cred(cred_dict={}):
    global cred_file
    global cred_dir

    if not Path.is_file(cred_file):
        cred_example_file_path = \
            Path(etld_lib_functions.qetl_code_dir, "qualys_etl", "etld_templates", ".etld_cred.yaml")
        destination_file_path = Path(cred_file)
        shutil.copy(str(cred_example_file_path), str(destination_file_path), follow_symlinks=True)
        cred_example_file = open(str(cred_example_file_path), "r", encoding='utf-8')
        cred_example = cred_example_file.read()
        cred_example_file.close()
        local_date = etld_lib_datetime.get_local_date()  # Add date updated to file
        cred_example = re.sub('\$DATE', local_date, cred_example)
        cred_file_example = open(str(cred_file), 'w', encoding='utf-8')
        cred_file_example.write(cred_example)
        cred_file_example.close()

    oschmod.set_mode(str(cred_file), "u+rw,u-x,go-rwx")
    if 'q_password' in os.environ.keys():
        cred = get_env_cred()
    else:
        cred = get_filesystem_cred()

    if 'bearer' in cred_dict.keys():
        cred['bearer'] = cred_dict['bearer']
    else:
        cred['bearer'] = ""

    cred['timer_obj'] = None

    return {'api_fqdn_server': cred['api_fqdn_server'],
            'gateway_fqdn_server': cred['gateway_fqdn_server'],
            'authorization': cred['authorization'],
            'username': cred['username'],
            'password': cred['password'],
            'bearer': cred['bearer'],
            'timer_obj': cred['timer_obj']}


def get_bearer_stored_in_env(update_bearer=True, cred=None):
    if cred is None:
        cred = {}

    max_age_of_bearer_token = 300

    timer_obj_update_bearer = False
    # if 'timer_obj' in cred.keys() and cred['timer_obj'] is not None:
    #     timer_obj: etld_lib_functions.TimerObj = cred['timer_obj']
    #     time_elapsed = timer_obj.get_time_elapsed()
    #     time_elapsed_flag = timer_obj.if_time_elapsed_past_max_time(max_time=max_age_of_bearer_token)
    #     if time_elapsed_flag:
    #         timer_obj_update_bearer = True
    #         timer_obj.reset_timer()
    #         etld_lib_functions.logger.info(f"time elapsed: {time_elapsed:,.0f} seconds")
    #     else:
    #         timer_obj_update_bearer = False
    #     cred['timer_obj'] = timer_obj
    # else:
    #     cred['timer_obj'] = etld_lib_functions.TimerObj()

    # TODO move away from file to timerobj for bearer token update.

    try:
        if update_bearer:
            cred = qualys_gateway_login_store_in_env(cred_dict=cred)
        if cred['bearer'] == "":
            cred = qualys_gateway_login_store_in_env(cred_dict=cred)
        elif Path(bearer_file).is_file():
            age_of_file = etld_lib_datetime.get_seconds_since_last_file_modification(Path(bearer_file))
            if age_of_file > max_age_of_bearer_token:
                cred = qualys_gateway_login_store_in_env(cred_dict=cred)
                etld_lib_functions.logger.info(f"Updated Bearer Token Successfully. "
                                               f"age_of_file={age_of_file}, max_age={max_age_of_bearer_token}")
        elif timer_obj_update_bearer:
            cred = qualys_gateway_login_store_in_env(cred_dict=cred)
        else:
            cred = qualys_gateway_login_store_in_env(cred_dict=cred)

    except Exception as e:
        etld_lib_functions.logger.error(f"               Credentials Dir:  {cred_dir}")
        etld_lib_functions.logger.error(f"              Credentials File:  {cred_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)

    return cred


def update_cred_dict(response, cred_dict) -> dict:
    with open(bearer_file, 'w', encoding='utf-8') as bearerfile:
        bearerfile.write(f"DUMMY ENTRY FOR TIME TRACKING OF BEARER AGE")
    bearer = f"Bearer {response.text}"
    cred_dict['bearer'] = bearer
    oschmod.set_mode(str(bearer_file), "u+rw,u-x,go-rwx")
    etld_lib_functions.logger.info(f"LOGIN - Qualys Gateway Login Success with user: {cred_dict['username']}")
    return cred_dict


def qualys_gateway_login_failed_message(response, cred_dict, log_level, message=""):
    if response != "":
        response_message = f", HTTP RESPONSE CODE: {response.status_code}"
    else:
        response_message = ""

    log_level(f"Qualys Gateway Login Failed with user: {cred_dict['username']} {response_message}")
    log_level(f"{message}")
    log_level(f"Credentials Env: username:{cred_dict['username']},"
              f"api_fqdn_server:{cred_dict['api_fqdn_server']},"
              f"gateway_fqdn_server:{cred_dict['gateway_fqdn_server']}")


def qualys_gateway_login_store_in_env(cred_dict={}):
    global login_gateway_failed
    global http_return_code

    login_gateway_failed = True
    if 'gateway_fqdn_server' in cred_dict:
        if cred_dict['gateway_fqdn_server'] is None:
            etld_lib_functions.logger.error(f"Please add gateway_fqdn_server credentials file or environment.")
            exit(1)

    # Login to Qualys, return bearer token.
    url = f"https://{cred_dict['gateway_fqdn_server']}/auth"  # Qualys Endpoint
    payload = {'token': 'true', 'password': cred_dict['password'], 'username': cred_dict['username'],
               'permissions': 'true'}
    payload = urlencode(payload, quote_via=quote_plus)

    headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
               'Content-Type': 'application/x-www-form-urlencoded',
               'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
    max_retries = 15
    sleep_time_requests_module_failed = 300
    sleep_time_503 = 10
    retry_number = 0
    for retry_number in range(max_retries):
        try:
            response = requests.request("POST", url, headers=headers, data=payload,
                                        verify=etld_lib_config.requests_module_tls_verify_status)
            http_return_code = response.status_code
            if response.status_code == 503:  # 503:  # Service Temporarily Unavailable
                qualys_gateway_login_failed_message(
                    message=f"Retry number {retry_number} for HTTP Response Code: {response.status_code} ",
                    response=response,
                    cred_dict=cred_dict,
                    log_level=etld_lib_functions.logger.warning)
                time.sleep(sleep_time_503)
            elif response.status_code == 201:
                cred_dict = update_cred_dict(response, cred_dict)
                login_gateway_failed = False
                break
            else:
                qualys_gateway_login_failed_message(
                    message=f"HTTP Response Code: {response.status_code} ",
                    response=response,
                    cred_dict=cred_dict,
                    log_level=etld_lib_functions.logger.error)
                exit(1)
        except requests.exceptions.RequestException as e:
            qualys_gateway_login_failed_message(
                message=f"Requests Module Failed {e}",
                response="",
                cred_dict=cred_dict,
                log_level=etld_lib_functions.logger.warning)
            etld_lib_functions.logger.warning(f"Exception: {e}")
            time.sleep(sleep_time_requests_module_failed)
    else:
        qualys_gateway_login_failed_message(response="ALL RETRIES FAILED",
                                            message=f"Retry number {retry_number} of {max_retries}",
                                            cred_dict=cred_dict,
                                            log_level=etld_lib_functions.logger.error)
        exit(3)

    return cred_dict


def test_qualys_login_logout_basic_auth() -> dict:

    def qualys_logout_test(login_status=None):
        if login_status is not None and login_status['cookie'] != "":
            cred_dict = get_cred()
            url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
            payload = {'action': 'logout'}
            headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
                       'Content-Type': 'application/x-www-form-urlencoded', 'Cookie': login_status['cookie'],
                       'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}
            try:
                response = requests.request("POST", url, headers=headers, data=payload,
                                            verify=etld_lib_config.requests_module_tls_verify_status)
                if response.status_code == 200:
                    etld_lib_functions.logger.info(f"LOGOUT - Qualys Logout Success with user: {cred_dict['username']}")
                else:
                    etld_lib_functions.logger.warning(f"LOGOUT FAILED - probably stale cookie, continue with warning")
            except Exception as e:
                etld_lib_functions.logger.warning(f"LOGOUT FAILED, probably connectivity issue, continue with warning")
                etld_lib_functions.logger.warning(f"Exception: {e}")
                pass

    def qualys_login_test() -> dict:
        login_status = {}
        login_status['login_failed'] = True
        login_status['cookie'] = ""

        cred_dict = get_cred()
        url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/session/"  # Qualys Endpoint
        payload = {'action': 'login', 'username': cred_dict['username'], 'password': cred_dict['password']}
        payload = urlencode(payload, quote_via=quote_plus)

        headers = {'X-Requested-With': f'qualysetl_v{qualys_etl.__version__}',
                   'Content-Type': 'application/x-www-form-urlencoded',
                   'User-Agent': f"qualysetl_v{qualys_etl.__version__}"}

        try:
            response = requests.request("POST", url, headers=headers, data=payload,
                                        verify=etld_lib_config.requests_module_tls_verify_status)
            login_status['response_status_code'] = response.status_code
            if response.status_code == 200:
                cookie_dict = response.cookies.get_dict()
                cookie = f"DWRSESSIONID={cookie_dict['DWRSESSIONID']}; QualysSession={cookie_dict['QualysSession']}"
                cookie = cookie.replace('\n', '').replace('\r', '')

                etld_lib_functions.logger.info(f"LOGIN - Qualys Login Success with user: {cred_dict['username']}")

                login_status['cookie'] = cookie
                login_status['login_failed'] = False
            else:
                etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user: {cred_dict['username']}")
                etld_lib_functions.logger.error(f"       HTTP {response.status_code}")
                etld_lib_functions.logger.error(f"       Verify Qualys username, password and "
                                                f"api_fqdn_server in Credentials File")
                etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
                etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
                etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
                exit(1)
        except requests.exceptions.RequestException as e:
            etld_lib_functions.logger.error(f"Fail - Qualys Login Failed with user")
            etld_lib_functions.logger.error(
                f"       Verify Qualys username, password and api_fqdn_server in Credentials File")
            etld_lib_functions.logger.error(f"             Credentials File: {cred_file}")
            etld_lib_functions.logger.error(f"             username:         {cred_dict['username']}")
            etld_lib_functions.logger.error(f"             api_fqdn_server:  {cred_dict['api_fqdn_server']}")
            etld_lib_functions.logger.error(f"Exception: {e}")
            exit(1)

        return login_status

    login_status = qualys_login_test()
    if login_status['login_failed']:
        pass
    else:
        qualys_logout_test(login_status)

    return login_status


def main():
    global cred_dir
    global cookie_file
    global bearer_file
    global cred_file
    global use_cookie
    global qualys_authentication_obj
    global platform_url

    platform_url = etld_lib_qualys_platform_identification.get_platform_url_dict()
    cred_dir = Path(etld_lib_config.qetl_user_cred_dir)  # Credentials Directory
    cookie_file = Path(cred_dir, ".etld_cookie")   # Never Used
    bearer_file = Path(cred_dir, ".etld_bearer")   # Dummy data, used to determine age of bearer, old replace by obj
    cred_file = Path(etld_lib_config.qetl_user_cred_file)  # YAML Format Qualys Credentials
    use_cookie = False
    # qualys_authentication_obj = etld_lib_authentication_objects.get_qualys_authentication_obj()
    # qualys_authentication_obj.get_current_bearer_token()
    # qualys_authentication_obj.get_authorization()
    # print(qualys_authentication_obj.__dir__())
    # print(qualys_authentication_obj.__dict__)

if __name__ == '__main__':
    platform_url = etld_lib_qualys_platform_identification.get_platform_url_dict()
    etld_lib_functions.main()
    etld_lib_config.main()
    main()
#    test_qualys_login_logout_basic_auth()



#!/usr/bin/env python3
import re
import sys
import argparse

def get_platform_identifier_dict(): # Update here whenever platform identification changes.
    platform_identifier_dict = {}
    platform_identifier_dict['_'] = {'qualysapi': 'qualysapi.qualys.com', 'gateway':'gateway.qg1.apps.qualys.com', 'pod': 'US_01'}
    platform_identifier_dict['2'] = {'qualysapi': 'qualysapi.qg2.apps.qualys.com','gateway': 'gateway.qg2.apps.qualys.com', 'pod': 'US_02'}
    platform_identifier_dict['3'] = {'qualysapi': 'qualysapi.qg3.apps.qualys.com','gateway': 'gateway.qg3.apps.qualys.com', 'pod': 'US_03'}
    platform_identifier_dict['6'] = {'qualysapi': 'qualysapi.qg4.apps.qualys.com','gateway': 'gateway.qg4.apps.qualys.com', 'pod': 'US_04'}
    platform_identifier_dict['-'] = {'qualysapi': 'qualysapi.qualys.eu','gateway': 'gateway.qg1.apps.qualys.eu', 'pod': 'EU_01'}
    platform_identifier_dict['5'] = {'qualysapi': 'qualysapi.qg2.apps.qualys.eu','gateway': 'gateway.qg2.apps.qualys.eu', 'pod': 'EU_02'}
    platform_identifier_dict['!'] = {'qualysapi': 'qualysapi.qg2.apps.qualys.eu','gateway': 'gateway.qg2.apps.qualys.eu', 'pod': 'EU_02'}
    platform_identifier_dict['B'] = {'qualysapi': 'qualysapi.qg3.apps.qualys.it','gateway': 'gateway.qg3.apps.qualys.it', 'pod': 'EU_03'}
    platform_identifier_dict['8'] = {'qualysapi': 'qualysapi.qg1.apps.qualys.in','gateway': 'gateway.qg1.apps.qualys.in', 'pod': 'IN_01'}
    platform_identifier_dict['9'] = {'qualysapi': 'qualysapi.qg1.apps.qualys.ca','gateway': 'gateway.qg1.apps.qualys.ca', 'pod': 'CA_01'}
    platform_identifier_dict['7'] = {'qualysapi': 'qualysapi.qg1.apps.qualys.ae','gateway': 'gateway.qg1.apps.qualys.eu', 'pod': 'AE_01'}
    platform_identifier_dict['1'] = {'qualysapi': 'qualysapi.qg1.apps.qualys.co.uk','gateway': 'gateway.qg1.apps.qualys.co.uk', 'pod': 'UK_01'}
    platform_identifier_dict['4'] = {'qualysapi': 'qualysapi.qg1.apps.qualys.com.au','gateway': 'gateway.qg1.apps.qualys.com.au', 'pod': 'AU_01'}
    platform_identifier_dict['A'] = {'qualysapi': 'qualysapi.qg1.apps.qualysksa.com','gateway': 'gateway.qg1.apps.qualysksa.com', 'pod': 'KSA_01'}
    return platform_identifier_dict


def get_platform_identification_with_userid(qualys_userid=""):
    pattern = re.compile(r'^([a-z]+)(.)(.*)$')
    userid = qualys_userid.strip()  # Remove any leading/trailing whitespace
    match = pattern.match(userid)
    if match:
        application_id, platform_identifier, application_user = match.groups()
        platform_identifier_dict = get_platform_identifier_dict()
        if platform_identifier in platform_identifier_dict:
            return platform_identifier_dict[platform_identifier]
        else:
            return None


def get_platform_identification_with_fqdn(fqdn=""):
    qualys_fqdn = fqdn.strip()  # Remove any leading/trailing whitespace
    platform_identifier_dict = get_platform_identifier_dict()
    for key in platform_identifier_dict:
        platform_qualysapi_fqdn = platform_identifier_dict[key]['qualysapi']
        platform_gateway_fqdn = platform_identifier_dict[key]['gateway']
        if qualys_fqdn == platform_qualysapi_fqdn:
            return platform_identifier_dict[key]
        elif qualys_fqdn == platform_gateway_fqdn:
            return platform_identifier_dict[key]
    return None

def get_platform_url_dict():
    platform_identifier_dict = get_platform_identifier_dict()
    platform_url_dict = {}
    for key in platform_identifier_dict:
        platform_qualysapi_fqdn = platform_identifier_dict[key]['qualysapi']
        platform_gateway_fqdn = platform_identifier_dict[key]['gateway']
        platform_url_dict[platform_qualysapi_fqdn] = platform_gateway_fqdn

    return platform_url_dict


def test_userid_from_stdin(input_obj=sys.stdin):
        for qualys_userid in input_obj:
            qualys_userid = qualys_userid.strip()  # Remove any leading/trailing whitespace
            platform_identification_dict = get_platform_identification_with_userid(qualys_userid)
            print(f"Lookup USERID..........:{qualys_userid.strip():40} {platform_identification_dict}")


def main():
    # Test ids sent to stdin
    qualys_userid_list = ['abc_123', 'abc2123', 'abc3123', 'abc6123', 'abc-123', 'abc5123', 'abc!123', 'abcB123', 'abc8123', 'abc9123', 'abc7123', 'abc1123', 'abc4123', 'abcA123']
    gateway_fqdn_list = []
    qualysapi_fqdn_list = []
    parser = argparse.ArgumentParser()
    parser.add_argument('function', help='Name of the function to run')
    args = parser.parse_args()

    if args.function == 'test_userid_from_stdin':
        test_userid_from_stdin()
    else:
        for qualys_userid in qualys_userid_list:
            platform_identification_dict = get_platform_identification_with_userid(qualys_userid)
            gateway_fqdn_list.append(platform_identification_dict['gateway'])
            qualysapi_fqdn_list.append(platform_identification_dict['qualysapi'])
            print(f"Lookup USERID..........:{qualys_userid.strip():40} {platform_identification_dict}")

        for gateway_fqdn in gateway_fqdn_list:
            platform_identification_dict = get_platform_identification_with_fqdn(gateway_fqdn)
            print(f"Lookup gateway fqdn....:{gateway_fqdn:40} {platform_identification_dict}")

        for qualysapi_fqdn in qualysapi_fqdn_list:
            platform_identification_dict = get_platform_identification_with_fqdn(qualysapi_fqdn)
            print(f"Lookup qualysapi fqdn..:{qualysapi_fqdn:40} {platform_identification_dict}")

# Call the function
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
import os
from pathlib import Path
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load
from qualys_etl.etld_lib import etld_lib_datetime


def host_list_detection_extract(
        xml_file: str, batch_of_host_ids: str, batch_number_str: str,
        qualys_headers_multiprocessing_dict, cred_dict: dict):

    begin_host_list_detection_04_extract(message=batch_number_str)
    authorization = cred_dict['authorization']  # Base64 user:password
    #url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/asset/host/vm/detection/"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.host_list_detection_api_endpoint}"

    payload = etld_lib_config.host_list_detection_api_payload
    payload_id_list = ",".join(str(x) for x in batch_of_host_ids)
    payload.update({'truncation_limit': '0', 'ids': payload_id_list})
    if isinstance(etld_lib_config.host_list_detection_payload_option, dict):
        payload.update(etld_lib_config.host_list_detection_payload_option)

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': authorization}

    print_payload = payload.copy()
    print_payload['ids'] = "TRUNCATED FOR LOG"
    etld_lib_functions.logger.info(f"api call     - {url}")
    etld_lib_functions.logger.info(f"api options  - {print_payload}")

    chunk_size_calc = etld_lib_config.host_list_detection_chunk_size_calc
    try_extract_max_count = etld_lib_config.host_list_detection_try_extract_max_count
    http_conn_timeout = etld_lib_config.host_list_detection_http_conn_timeout
    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        output_file=xml_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_multiprocessing_dict,
        batch_number_formatted=batch_number_str,
        compression_method=etld_lib_config.host_list_detection_open_file_compression_method)

    end_host_list_detection_04_extract(message=batch_number_str)


def get_qualys_limits_from_host_list_detection(qualys_headers_multiprocessing_dict, cred_dict):

    authorization = cred_dict['authorization']  # Base64 user:password
    #url = f"https://{cred_dict['api_fqdn_server']}/api/2.0/fo/asset/host/vm/detection/"
    url = f"https://{cred_dict['api_fqdn_server']}{etld_lib_config.host_list_detection_api_endpoint}"

    payload = {'action': 'list',
               'truncation_limit': '1',
               }

    headers = {'X-Requested-With': 'qualysetl', 'Authorization': authorization}

    xml_file = Path(os.devnull)
    chunk_size_calc = etld_lib_config.host_list_detection_chunk_size_calc
    try_extract_max_count = etld_lib_config.host_list_detection_try_extract_max_count
    http_conn_timeout = etld_lib_config.asset_inventory_http_conn_timeout
    batch_number_str = "check_headers"
    etld_lib_extract_transform_load.extract_qualys(
        try_extract_max_count=try_extract_max_count,
        url=url,
        headers=headers,
        payload=payload,
        http_conn_timeout=http_conn_timeout,
        chunk_size_calc=chunk_size_calc,
        output_file=xml_file,
        cred_dict=cred_dict,
        qualys_headers_multiprocessing_dict=qualys_headers_multiprocessing_dict,
        batch_number_formatted=batch_number_str)


def end_message_info(url, xml_file_utc_run_datetime, xml_file):
    etld_lib_functions.log_file_info(url, 'url')
    etld_lib_functions.logger.info(f"Run Date: {xml_file_utc_run_datetime}")
    etld_lib_functions.log_file_info(xml_file)


def begin_host_list_detection_04_extract(message=""):
    etld_lib_functions.logger.info(f"start {message}")


#def end_host_list_detection_04_extract(url, xml_file_utc_run_datetime, xml_file, message=""):
#    end_message_info(url, xml_file_utc_run_datetime, xml_file)
#    etld_lib_functions.logger.info(f"end   {message}")

def end_host_list_detection_04_extract(message=""):
    etld_lib_functions.logger.info(f"end   {message}")


def main(args=None):
    test_one_batch()


def test_one_batch():
    # TODO get list of batch id's for testing.
    xml_file_utc_run_datetime = etld_lib_datetime.get_utc_datetime_qualys_format()
    file_info_dict = \
        etld_lib_config.prepare_extract_batch_file_name(
            next_batch_number_str="batch_000001",
            next_batch_date=xml_file_utc_run_datetime,
            extract_dir=etld_lib_config.host_list_detection_extract_dir,
            file_name_type="host_list_detection",
            file_name_option="vm_processed_after",
            file_name_option_date=etld_lib_config.host_list_detection_vm_processed_after,
            compression_method=etld_lib_config.host_list_detection_open_file_compression_method
        )

    #cred_dict=etld_lib_credentials.get_cred(cred_dict={})
    credentials_dict = etld_lib_authentication_objects.qualys_authentication_obj.get_credentials_dict()
    host_list_detection_extract(
        xml_file=file_info_dict['next_file_path'],
        batch_of_host_ids="",
        batch_number_str=file_info_dict['next_batch_number_str'],
        qualys_headers_multiprocessing_dict=None,
        cred_dict=credentials_dict
    )


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='host_list_detection_04_extract')
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()






#!/usr/bin/env python3
import json
from pathlib import Path
import re
from multiprocessing import Process, Queue
import time

from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_datetime
from qualys_etl.etld_lib import etld_lib_extract_transform_load
from qualys_etl.etld_lib import etld_lib_sqlite_tables
from qualys_etl.etld_lib import etld_lib_csv_distribution


def transform_epoch_dates(item):
    item['sensor_lastVMScanDate'] = ""
    item['sensor_lastComplianceScanDate'] = ""
    item['sensor_lastFullScanDate'] = ""
    item['agent_lastActivityDate'] = ""
    item['agent_lastCheckedInDate'] = ""
    item['agent_lastInventoryDate'] = ""
    item['inventory_createdDate'] = ""
    item['inventory_lastUpdatedDate'] = ""
    # Added 2022-12-16
    item['sensor_lastPcScanDateAgent'] = ""
    item['sensor_lastPcScanDateScanner'] = ""
    item['sensor_lastVmScanDateAgent'] = ""
    item['sensor_lastVmScanDateScanner'] = ""

    if 'sensor' in item.keys() and item['sensor'] is not None:
        if 'lastVMScan' in item['sensor']:
            item['sensor_lastVMScanDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastVMScan'])
        if 'lastComplianceScan' in item['sensor']:
            item['sensor_lastComplianceScanDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastComplianceScan'])
        if 'lastFullScan' in item['sensor']:
            item['sensor_lastFullScanDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastFullScan'])
        if 'lastPcScanDateAgent' in item['sensor']:
            item['sensor_lastPcScanDateAgent'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastPcScanDateAgent'])
        if 'lastPcScanDateScanner' in item['sensor']:
            item['sensor_lastPcScanDateScanner'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastPcScanDateScanner'])
        if 'lastVmScanDateAgent' in item['sensor']:
            item['sensor_lastVmScanDateAgent'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastVmScanDateAgent'])
        if 'lastVmScanDateScanner' in item['sensor']:
            item['sensor_lastVmScanDateScanner'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['sensor']['lastVmScanDateScanner'])

    if 'agent' in item.keys() and item['agent'] is not None:
        if 'lastActivity' in item['agent']:
            item['agent_lastActivityDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['agent']['lastActivity'])
        if 'lastCheckedIn' in item['agent']:
            item['agent_lastCheckedInDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['agent']['lastCheckedIn'])
        if 'lastInventory' in item['agent']:
            item['agent_lastInventoryDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['agent']['lastInventory'])

    if 'inventory' in item.keys() and item['inventory'] is not None:
        if 'created' in item['inventory']:
            item['inventory_createdDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['inventory']['created'])
        if 'lastUpdated' in item['inventory']:
            item['inventory_lastUpdatedDate'] = \
                etld_lib_datetime.get_datetime_str_from_epoch_milli_sec(item['inventory']['lastUpdated'])
    return item


def insert_one_row_into_table_q_pcrs(
        sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
        table_name: str,
        table_fields: list,
        table_field_types: list,
        asset_item_dict: dict,
        counter_obj,
        counter_obj_duplicates,
        batch_date: str,
        batch_number: str,
        batch_name: str,
        csv_distribution_dict=None
):

    # def prepare_pcrs_field(field_data, field_name_tmp):
    #     if field_data is None:
    #         field_data = ""
    #     elif 'Date' in field_name_tmp:
    #         field_data = field_data.replace("T", " ").replace("Z", "")
    #         field_data = re.sub("\\..*$", "", field_data)
    #     elif 'lastBoot' in field_name_tmp:
    #         field_data = field_data.replace("T", " ").replace("Z", "")
    #         field_data = re.sub("\\..*$", "", field_data)
    #     elif isinstance(field_data, int):
    #         field_data = str(field_data)
    #     elif not isinstance(field_data, str):
    #         field_data = json.dumps(field_data)
    #
    #     return field_data
    #
    # row_in_sqlite_form = []
    # for field_name in table_fields:  # Iterate through expected columns (contract)
    #     if field_name in asset_item_dict.keys():  # Iterate through columns found in dictionary
    #         asset_item_dict[field_name] = \
    #             sqlite_obj.prepare_database_field_pcrs(asset_item_dict[field_name],
    #                                                               field_name,
    #                                                               csv_column_types=table_field_types)
    #         row_in_sqlite_form.append(asset_item_dict[field_name])
    #     else:
    #         row_in_sqlite_form.append("")  # Ensure blank is added to each required empty field

    row_in_sqlite_form = sqlite_obj.prepare_database_row_pcrs(
        item_dict=asset_item_dict,
        csv_columns=table_fields,
        csv_column_types=table_field_types,
        batch_name=batch_name,
        batch_date=batch_date,
        batch_number=batch_number)

    result = sqlite_obj.insert_unique_row_ignore_duplicates(table_name, row_in_sqlite_form)
    if result is True:
        counter_obj.update_counter_and_display_to_log()
        # WRITE CSV ROW
        if etld_lib_config.pcrs_distribution_csv_flag and csv_distribution_dict is not None:
            #
            # Write one row to Q_Asset_Inventory CSV File
            #
            qids_csv_obj: etld_lib_csv_distribution.CsvDistribution = \
                csv_distribution_dict['q_pcrs_csv_obj']
            qids_csv_obj.write_csv_row_to_file_handle(row_in_sqlite_form)
    else:
        counter_obj_duplicates.update_counter_and_display_to_log()


def insert_one_row_into_table_q_pcrs_software_assetid(
        asset_item_dict: dict,
        sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
        counter_obj,
        batch_date: str,
        batch_number: str,
        batch_name: str
):

    if asset_item_dict['softwareListData'] is not None:
        for software_item in asset_item_dict['softwareListData']['software']:
            if 'fullName' in software_item:
                assetid = str(asset_item_dict['assetId'])
                fullname = software_item['fullName']
                row = {'assetId': assetid, 'fullName': fullname}
                sqlite_row = []
                for field in etld_lib_config.pcrs_software_assetid_csv_columns():
                    if field in row.keys():
                        sqlite_row.append(row[field])
                    else:
                        sqlite_row.append("")

                result = sqlite_obj.insert_unique_row_ignore_duplicates(
                    etld_lib_config.pcrs_table_name_software_assetid,
                    sqlite_row)
                if result is True:
                    counter_obj.update_counter_and_display_to_log()
                    # WRITE CSV ROW
                else:
                    pass
                    #etld_lib_functions.logger.info("found dups in software assetid")


def insert_one_row_into_table_q_pcrs_software_os_unique(
        asset_item_dict: dict,
        sqlite_obj: etld_lib_sqlite_tables.SqliteObj,
        counter_obj,
        batch_date: str,
        batch_number: str,
        batch_name: str,
):

    if asset_item_dict['softwareListData'] is not None:
        for software_item in asset_item_dict['softwareListData']['software']:
            if 'fullName' in software_item:
                fullname = str(software_item['fullName'])
                osname = ""
                isignored = ""
                ignoredreason = ""
                category = ""
                if 'operatingSystem' in asset_item_dict:
                    if 'osName' in asset_item_dict['operatingSystem']:
                        osname = str(asset_item_dict['operatingSystem']['osName'])
                if 'lifecycle' in software_item:
                    lifecycle = software_item['lifecycle']
                    if lifecycle is None:
                        lifecycle = {}
                else:
                    lifecycle = {}
                if 'isIgnored' in software_item:
                    isignored = str(software_item['isIgnored'])
                if 'ignoredReason' in software_item:
                    ignoredreason = str(software_item['ignoredReason'])
                if 'category' in software_item:
                    category = \
                        f"{software_item['category']} | {software_item['category']} | {software_item['category']}"

                row = {'fullName': fullname, 'osName': osname,
                       'isIgnored': isignored, 'ignoredReason': ignoredreason, 'category': category,
                       }
                row.update(lifecycle)
                sqlite_row = []
                for field in etld_lib_config.pcrs_software_os_unique_csv_columns():
                    if field in row.keys():
                        sqlite_row.append(row[field])
                    else:
                        sqlite_row.append("")

                result = sqlite_obj.insert_unique_row_ignore_duplicates(
                    etld_lib_config.pcrs_table_name_software_os_unique,
                    sqlite_row)
                if result is True:
                    counter_obj.update_counter_and_display_to_log()
                    # WRITE CSV ROW


def drop_and_create_all_tables(sqlite_obj):
    sqlite_obj.drop_and_recreate_table(
        table_name=etld_lib_config.pcrs_table_name,
        csv_columns=etld_lib_config.pcrs_csv_columns(),
        csv_column_types=etld_lib_config.pcrs_csv_column_types(),
        key='assetId')

    sqlite_obj.drop_and_recreate_table(
        table_name=etld_lib_config.pcrs_status_table_name,
        csv_columns=etld_lib_config.status_table_csv_columns(),
        csv_column_types=etld_lib_config.status_table_csv_column_types(),
        key='STATUS_NAME')

    sqlite_obj.drop_and_recreate_table(
        table_name=etld_lib_config.pcrs_table_name_software_os_unique,
        csv_columns=etld_lib_config.pcrs_software_os_unique_csv_columns(),
        csv_column_types=etld_lib_config.pcrs_software_os_unique_csv_column_types(),
        key=['fullName', 'osName'])

    sqlite_obj.drop_and_recreate_table(
        table_name=etld_lib_config.pcrs_table_name_software_assetid,
        csv_columns=etld_lib_config.pcrs_software_assetid_csv_columns(),
        csv_column_types=etld_lib_config.pcrs_software_assetid_csv_column_types(),
        key=['assetId', 'fullName'])


def create_counter_objects(database_type='sqlite'):
    counter_obj_pcrs = etld_lib_functions.DisplayCounterToLog(
        display_counter_at=25000,
        logger_func=etld_lib_functions.logger.info,
        display_counter_log_message=f"rows added to {database_type} "
                                    f"table {etld_lib_config.pcrs_table_name}")
    counter_obj_pcrs_duplicates = etld_lib_functions.DisplayCounterToLog(
        display_counter_at=10000,
        logger_func=etld_lib_functions.logger.info,
        display_counter_log_message=f"duplicate rows bypassed, not written to {database_type} "
                                    f"table {etld_lib_config.pcrs_table_name}")
    counter_obj_software_os = etld_lib_functions.DisplayCounterToLog(
        display_counter_at=25000,
        logger_func=etld_lib_functions.logger.info,
        display_counter_log_message=f"rows added to {database_type} "
                                    f"table {etld_lib_config.pcrs_table_name_software_os_unique}")
    counter_obj_software_assetid = etld_lib_functions.DisplayCounterToLog(
        display_counter_at=25000,
        logger_func=etld_lib_functions.logger.info,
        display_counter_log_message=f"rows added to {database_type} "
                                    f"table {etld_lib_config.pcrs_table_name_software_assetid}")
    counter_objects = {'counter_obj_pcrs': counter_obj_pcrs,
                       'counter_obj_pcrs_duplicates': counter_obj_pcrs_duplicates,
                       'counter_obj_software_os': counter_obj_software_os,
                       'counter_obj_software_assetid': counter_obj_software_assetid}

    return counter_objects


def insert_one_asset_into_multiple_tables(asset_item, sqlite_obj, counter_objects, csv_distribution_dict=None,
                                          batch_date="", batch_number="", batch_name=""):
    item = transform_epoch_dates(asset_item)
    insert_one_row_into_table_q_pcrs_software_os_unique(
        asset_item_dict=item,
        sqlite_obj=sqlite_obj,
        counter_obj=counter_objects['counter_obj_software_os'],
        batch_date=batch_date,
        batch_number=batch_number,
        batch_name=batch_name
    )
    insert_one_row_into_table_q_pcrs_software_assetid(
        asset_item_dict=item,
        sqlite_obj=sqlite_obj,
        counter_obj=counter_objects['counter_obj_software_assetid'],
        batch_date=batch_date,
        batch_number=batch_number,
        batch_name=batch_name
    )
    insert_one_row_into_table_q_pcrs(
        sqlite_obj=sqlite_obj,
        table_name=etld_lib_config.pcrs_table_name,
        table_fields=etld_lib_config.pcrs_csv_columns(),
        table_field_types=etld_lib_config.pcrs_csv_column_types(),
        asset_item_dict=item,
        counter_obj=counter_objects['counter_obj_pcrs'],
        counter_obj_duplicates=counter_objects['counter_obj_pcrs_duplicates'],
        csv_distribution_dict=csv_distribution_dict,
        batch_date=batch_date,
        batch_number=batch_number,
        batch_name=batch_name
    )


def transform_and_load_queue_of_json_files_into_sqlite(queue_of_json_files,):
    begin_pcrs_05_transform_load(message="queue_of_json_files_through_multiprocesssing")
    file_path = "EXCEPTION"
    try:
        sqlite_pcrs = etld_lib_sqlite_tables.SqliteObj(etld_lib_config.pcrs_sqlite_file)
        drop_and_create_all_tables(sqlite_pcrs)
        counter_objects = create_counter_objects()
        while True:
            time.sleep(2)
            file_path = queue_of_json_files.get()
            if file_path == 'BEGIN':
                etld_lib_functions.logger.info(f"Found BEGIN of Queue.")
                break

        while True:
            time.sleep(2)
            file_path = queue_of_json_files.get()
            if file_path == 'END':
                etld_lib_functions.logger.info(f"Found END of Queue.")
                break

            with etld_lib_config.pcrs_open_file_compression_method(str(file_path), "rt", encoding='utf-8') \
                    as read_file:
                batch_name = etld_lib_extract_transform_load.get_batch_name_from_filename(file_name=file_path)
                batch_number = etld_lib_extract_transform_load.get_batch_number_from_filename(file_name=file_path)
                batch_date = etld_lib_extract_transform_load.get_batch_date_from_filename(file_name=file_path)
                etld_lib_functions.logger.info(f"Received batch file from multiprocessing Queue: {Path(file_path).name}")

                sqlite_pcrs.update_status_table(
                        batch_date=batch_date, batch_number=batch_number,
                        total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                        status_table_name=etld_lib_config.pcrs_status_table_name,
                        status_table_columns=etld_lib_config.status_table_csv_columns(),
                        status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                        status_name_column=etld_lib_config.pcrs_table_name, status_column='begin')

                all_items = json.load(read_file)
                csv_distribution_dict = {}
                if etld_lib_config.host_list_detection_distribution_csv_flag:
                    q_pcrs_csv_info =  etld_lib_csv_distribution.get_csv_info_dict(
                        batch_date=batch_date,
                        batch_number=batch_number,
                        table_name=etld_lib_config.pcrs_table_name,
                        csv_file_open_method=etld_lib_config.pcrs_open_file_compression_method,
                        distribution_dir=etld_lib_config.pcrs_distribution_dir
                    )
                    with q_pcrs_csv_info['compression_method'](
                            q_pcrs_csv_info['target_tmp_csv_file_path'], 'wt', newline='') as \
                            q_pcrs_target_tmp_csv_file_handle:
                        q_pcrs_csv_info['csv_obj'].set_csv_writer_with_file_handle(
                            q_pcrs_target_tmp_csv_file_handle)
                        csv_distribution_dict['q_pcrs_csv_obj'] = q_pcrs_csv_info['csv_obj']
                        for item in all_items['assetListData']['asset']:
                            insert_one_asset_into_multiple_tables(
                                item, sqlite_pcrs, counter_objects,
                                csv_distribution_dict=csv_distribution_dict,
                                batch_date=batch_date, batch_number=batch_number, batch_name=batch_name
                            )
                    Path(q_pcrs_csv_info['target_tmp_csv_file_path']).\
                            rename(q_pcrs_csv_info['target_csv_file_path'])
                else:
                    for item in all_items['assetListData']['asset']:
                        insert_one_asset_into_multiple_tables(
                            item, sqlite_pcrs, counter_objects, csv_distribution_dict=None,
                            batch_date=batch_date, batch_number=batch_number, batch_name=batch_name
                        )

                sqlite_pcrs.commit_changes()

                sqlite_pcrs.update_status_table(
                        batch_date=batch_date, batch_number=batch_number,
                        total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                        status_table_name=etld_lib_config.pcrs_status_table_name,
                        status_table_columns=etld_lib_config.status_table_csv_columns(),
                        status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                        status_name_column=etld_lib_config.pcrs_table_name, status_column='end')

                etld_lib_functions.logger.info(
                    f"Committed batch file from multiprocessing Queue into Database: {batch_name}")

        etld_lib_functions.logger.info(f"Completed processing Queue of files")

        sqlite_pcrs.update_status_table(
                batch_date=batch_date, batch_number=batch_number,
                total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                status_table_name=etld_lib_config.pcrs_status_table_name,
                status_table_columns=etld_lib_config.status_table_csv_columns(),
                status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                status_name_column=etld_lib_config.pcrs_table_name, status_column='final')

        sqlite_pcrs.update_status_table(
            batch_date=batch_date,
            batch_number=0,
            total_rows_added_to_database=0,
            status_table_name=etld_lib_config.pcrs_status_table_name,
            status_table_columns=etld_lib_config.status_table_csv_columns(),
            status_table_column_types=etld_lib_config.status_table_csv_column_types(),
            status_name_column='ALL_TABLES_LOADED_SUCCESSFULLY',
            status_column='final')

        sqlite_pcrs.commit_changes()
        sqlite_pcrs.close_connection()
        counter_objects['counter_obj_pcrs_duplicates'].display_final_counter_to_log()
        counter_objects['counter_obj_pcrs'].display_final_counter_to_log()
        counter_objects['counter_obj_software_os'].display_final_counter_to_log()
        counter_objects['counter_obj_software_assetid'].display_final_counter_to_log()

    except Exception as e:
        etld_lib_functions.logger.error(f"Exception: {e}")
        etld_lib_functions.logger.error(f"Potential JSON File corruption detected: {file_path}")
        exit(1)
    end_pcrs_05_transform_load(message="queue_of_json_files_through_multiprocesssing")


def begin_pcrs_05_transform_load(message=""):
    etld_lib_functions.logger.info(f"start {message}")


def end_pcrs_05_transform_load(message=""):
    etld_lib_functions.logger.info(f"end  {message}")


def transform_and_load_all_json_files_into_sqlite():
    begin_pcrs_05_transform_load(message="directory_listing_of_json_files")
    json_file = "EXCEPTION"
    try:
        sqlite_pcrs = etld_lib_sqlite_tables.SqliteObj(etld_lib_config.pcrs_sqlite_file)
        drop_and_create_all_tables(sqlite_pcrs)
        counter_objects = create_counter_objects()
        json_file_list = \
            sorted(Path(etld_lib_config.pcrs_extract_dir).glob(
                etld_lib_config.pcrs_extract_dir_file_search_blob))
        for json_file in json_file_list:
            with etld_lib_config.pcrs_open_file_compression_method(str(json_file), "rt", encoding='utf-8') \
                    as read_file:

                batch_name = etld_lib_extract_transform_load.get_batch_name_from_filename(file_name=json_file)
                batch_number = etld_lib_extract_transform_load.get_batch_number_from_filename(file_name=json_file)
                batch_date = etld_lib_extract_transform_load.get_batch_date_from_filename(file_name=json_file)
                etld_lib_functions.logger.info(f"Received batch file from directory: {batch_name}")

                sqlite_pcrs.update_status_table(
                        batch_date=batch_date, batch_number=batch_number,
                        total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                        status_table_name=etld_lib_config.pcrs_status_table_name,
                        status_table_columns=etld_lib_config.status_table_csv_columns(),
                        status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                        status_name_column=etld_lib_config.pcrs_table_name, status_column='begin')

                all_items = json.load(read_file)
                # Open CSV File
                for item in all_items['assetListData']['asset']:
                    insert_one_asset_into_multiple_tables(
                        item,
                        sqlite_pcrs,
                        counter_objects,
                        csv_distribution_dict=None,
                        batch_date=batch_date,
                        batch_number=batch_number,
                        batch_name=batch_name
                    )

                etld_lib_functions.logger.info(
                    f"Added batch file from directory into Database: {batch_name}")

                sqlite_pcrs.update_status_table(
                        batch_date=batch_date, batch_number=batch_number,
                        total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                        status_table_name=etld_lib_config.pcrs_status_table_name,
                        status_table_columns=etld_lib_config.status_table_csv_columns(),
                        status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                        status_name_column=etld_lib_config.pcrs_table_name, status_column='end')

        sqlite_pcrs.update_status_table(
                batch_date=batch_date, batch_number=batch_number,
                total_rows_added_to_database=counter_objects['counter_obj_pcrs'].get_counter(),
                status_table_name=etld_lib_config.pcrs_status_table_name,
                status_table_columns=etld_lib_config.status_table_csv_columns(),
                status_table_column_types=etld_lib_config.status_table_csv_column_types(),
                status_name_column=etld_lib_config.pcrs_table_name, status_column='final')

        sqlite_pcrs.update_status_table(
            batch_date=batch_date,
            batch_number=0,
            total_rows_added_to_database=0,
            status_table_name=etld_lib_config.pcrs_status_table_name,
            status_table_columns=etld_lib_config.status_table_csv_columns(),
            status_table_column_types=etld_lib_config.status_table_csv_column_types(),
            status_name_column='ALL_TABLES_LOADED_SUCCESSFULLY',
            status_column='final')

        etld_lib_functions.logger.info(f"Completed processing directory of files")
        sqlite_pcrs.commit_changes()
        sqlite_pcrs.close_connection()
        counter_objects['counter_obj_pcrs_duplicates'].display_final_counter_to_log()
        counter_objects['counter_obj_pcrs'].display_final_counter_to_log()
        counter_objects['counter_obj_software_os'].display_final_counter_to_log()
        counter_objects['counter_obj_software_assetid'].display_final_counter_to_log()

    except Exception as e:
        etld_lib_functions.logger.error(f"JSON File Issue: {json_file}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
    end_pcrs_05_transform_load(message="directory_listing_of_json_files")


def spawn_multiprocessing_queue_to_transform_and_load_json_files_into_sqlite():
    queue_of_file_paths = Queue()
    queue_process = \
        Process(target=transform_and_load_queue_of_json_files_into_sqlite, args=(queue_of_file_paths,))
    queue_process.daemon = True
    queue_process.start()
    batch_queue_of_file_paths = queue_of_file_paths
    batch_queue_process = queue_process

    return batch_queue_of_file_paths, batch_queue_process


def main():
    transform_and_load_all_json_files_into_sqlite()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='pcrs_05_transform_load_json_to_sqlite')
    etld_lib_config.main()
    etld_lib_config.pcrs_json_to_sqlite_via_multiprocessing = False
    main()



#!/usr/bin/env python3
import sys
import timeit
from pathlib import Path
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_config
#from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects

from qualys_etl.etld_knowledgebase import knowledgebase_03_extract_controller
from qualys_etl.etld_knowledgebase import knowledgebase_04_extract
from qualys_etl.etld_knowledgebase import knowledgebase_05_transform_load_xml_to_sqlite
from qualys_etl.etld_knowledgebase import knowledgebase_06_distribution
#import traceback
import time

global start_time
global stop_time


def knowledgebase_03_extract_controller_wrapper(module_function=knowledgebase_03_extract_controller, message=""):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def knowledgebase_04_extract_wrapper(module_function=knowledgebase_04_extract, message=""):
    etld_lib_functions.logger.info(f"start {module_function} {message}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def knowledgebase_05_transform_load_xml_to_sqlite_wrapper(module_function=knowledgebase_05_transform_load_xml_to_sqlite):
    etld_lib_functions.logger.info(f"start {module_function}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def knowledgebase_06_distribution_wrapper(module_function=knowledgebase_06_distribution):
    etld_lib_functions.logger.info(f"start {module_function}")
    module_function.main()
    etld_lib_functions.logger.info(f"end   {module_function}")


def begin_knowledgebase_02_workflow_manager():
    global start_time
    start_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"__start__ knowledgebase_etl_workflow {str(sys.argv)}")
    etld_lib_functions.logger.info(f"data directory: {etld_lib_config.qetl_user_data_dir}")


def end_knowledgebase_02_workflow_manager():
    global start_time
    global stop_time

    stop_time = timeit.default_timer()
    etld_lib_functions.logger.info(f"runtime for knowledgebase_etl_workflow in seconds: {stop_time - start_time:,}")
    etld_lib_functions.logger.info(f"__end__ knowledgebase_etl_workflow {str(sys.argv)}")


def knowledgebase_etl_workflow():
    try:
        begin_knowledgebase_02_workflow_manager()
        knowledgebase_03_extract_controller_wrapper()
        knowledgebase_05_transform_load_xml_to_sqlite_wrapper()
        knowledgebase_06_distribution_wrapper()
        end_knowledgebase_02_workflow_manager()
    except Exception as e:
        time.sleep(10)
        etld_lib_functions.logger.error(f"Error knowledgebase_02_workflow_manager - knowledgebase_etl_workflow, "
                                        f"please investigate {sys.argv}")
        etld_lib_functions.logger.error(f"Exception: {e}")
        exit(1)
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except KeyboardInterrupt:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error knowledgebase_02_workflow_manager - knowledgebase_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a KeyboardInterrupt, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)
    # except SystemExit:
    #     time.sleep(10)
    #     etld_lib_functions.logger.error(f"Error knowledgebase_02_workflow_manager - knowledgebase_etl_workflow, "
    #                                     f"please investigate {sys.argv}")
    #     etld_lib_functions.logger.error("Caught a SystemExit, performing cleanup...")
    #     formatted_lines = traceback.format_exc().splitlines()
    #     etld_lib_functions.logger.error(f"TRACEBACK: {formatted_lines}")
    #     exit(1)


def main():
    knowledgebase_etl_workflow()


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name="knowledgebase_02_workflow_manager")
    etld_lib_config.main()
    #etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    main()



#!/usr/bin/env python3
import time
import sys
import json
from pathlib import Path
import re
import gzip
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_credentials
from qualys_etl.etld_lib import etld_lib_authentication_objects
from qualys_etl.etld_lib import etld_lib_functions
from qualys_etl.etld_lib import etld_lib_extract_transform_load
from qualys_etl.etld_lib import etld_lib_datetime
from qualys_etl.etld_knowledgebase import knowledgebase_02_workflow_manager
from qualys_etl.etld_host_list import host_list_02_workflow_manager
from qualys_etl.etld_host_list_detection import host_list_detection_02_workflow_manager
from qualys_etl.etld_asset_inventory import asset_inventory_02_workflow_manager
from qualys_etl.etld_was import was_02_workflow_manager
from qualys_etl.etld_was import was_04_extract_from_qualys


def validate_json_file(open_file_method=open, file_path: Path = Path()):
    last_characters = []
    validation_type = 'tail file'

    def get_compression_method(file_path):
        # https://www.garykessler.net/library/file_sigs.html for magic
        with open(file_path, 'rb') as f:
            file_bytes = f.read(3)
            if file_bytes.startswith(b'\x1f\x8b\x08'):
                return gzip.open
            else:
                return open

    open_file_method = get_compression_method(file_path)
    with open_file_method(file_path, "rt", encoding='utf-8') as read_file:
        last_characters_test_passed = False
        try:
            # light validation detecting closure of json with curly brace closure
            last_characters = tail_file_by_last_characters(open_file_method=open_file_method,
                                                           file_path=file_path,
                                                           number_of_characters=25)
            validate_last_characters_are_str = '}'
            if last_characters[-1:] == validate_last_characters_are_str:
                last_characters_test_passed = True
        except Exception as e:
            last_characters_test_passed = False

        try:
            if last_characters_test_passed is False:
                validation_type = 'json.load file'
                json.load(read_file)
        except Exception as e:
            raise Exception(f"WARNING: MALFORMED JSON, COULD NOT json.load batch file, "
                            f"validation_test_type={validation_type}, "
                            f"directory={Path(file_path).parent.name}, "
                            f"batch file={etld_lib_extract_transform_load.get_batch_name_from_filename(file_path)}, "
                            f"Exception={e}"
                            )
        # TODO hasMoreRecords check for key and retry if key is missing.
        # CSAM etl_workflow_validation_type is ...
        # WAS etl_workflow_validation_type is
        tail_file_test=last_characters.replace('|', '').replace("\n", "")
        etld_lib_functions.logger.info(f"PASS: JSON VALIDATED, "
                                       f"validation_test_type={validation_type}, "
                                       f"directory={Path(file_path).parent.name}, "
                                       f"batch file={etld_lib_extract_transform_load.get_batch_name_from_filename(file_path)}, "
                                       f"tail file={tail_file_test}")


def validate_extract_directories_json_files_downloaded():
    json_dir_list = []
    json_dir_list.append(Path(etld_lib_config.host_list_extract_dir))
    json_dir_list.append(Path(etld_lib_config.host_list_detection_extract_dir))
    json_dir_list.append(Path(etld_lib_config.kb_extract_dir))
    json_dir_list.append(Path(etld_lib_config.asset_inventory_extract_dir))
    json_dir_list.append(Path(etld_lib_config.was_extract_dir))
    json_dir_search_glob = '*.json.gz'

    for json_dir in json_dir_list:
        json_file_list = sorted(Path(json_dir).glob(json_dir_search_glob))
        for json_file_path in json_file_list:
            try:
                validate_json_file(open_file_method=gzip.open,
                                   file_path=json_file_path)
            except Exception as e:
                etld_lib_functions.logger.error(f'Exception {e}')


def tail_file_by_last_characters(open_file_method=open, file_path: Path = Path(), number_of_characters: int = 1000):
    file_position = number_of_characters
    last_lines_from_file = []
    file_length_in_utf_8_characters = 0
    with open_file_method(file_path, mode="rt", encoding='utf-8') as f:
        try:
            f.seek(0, 2)
            file_length_in_utf_8_characters = f.tell()
            if file_length_in_utf_8_characters <= number_of_characters:
                file_position = 0
            else:
                file_position = file_length_in_utf_8_characters - number_of_characters

            f.seek(file_position, 0)
            last_characters_list = []
            while True:
                char = f.read(1)
                if not char:
                    break
                last_characters_list.append(char)
        except IOError as ioe:
            raise Exception(ioe)
        finally:
            last_lines_from_file = ''.join(last_characters_list)
            return last_lines_from_file


def tail_file_by_lines(open_file_method=open, file_path: Path = Path(), number_of_lines: int = 10):
    file_position = number_of_lines + 1
    last_lines_from_file = []
    with open_file_method(file_path, mode="rt", encoding='utf-8') as f:
        while len(last_lines_from_file) <= number_of_lines:
            try:
                f.seek(-file_position, 2)
            except IOError:
                f.seek(0)
                break
            finally:
                last_lines_from_file = list(f)
            file_position *= 2
    return last_lines_from_file[-number_of_lines:]


def validate_xml_is_closed_properly(test_by='xml_characters',
                                    open_file_method=gzip.open,
                                    file_path: Path = Path(),
                                    number_of_lines: int = 10,
                                    number_of_characters: int = 1000):
    batch = re.sub('^.*batch', 'batch', str(file_path))
    directory_name = str(file_path.parent.name)
    xml_lines = []
    xml_characters_list = []
    try:
        if 'characters' in test_by:
            xml_lines = tail_file_by_last_characters(open_file_method, file_path, number_of_characters)
        else:
            xml_lines = tail_file_by_lines(open_file_method, file_path, number_of_lines)
    except Exception as e:
        xml_line = ''.join(xml_lines.replace('\n', '').replace('|', ''))
        raise Exception(f"WARNING: CANNOT READ, CREATION MAY BE IN PROGRESS "
                        f"test_by={test_by} "
                        f"directory={directory_name} "
                        f"batch file={batch}, "
                        f"Exception={e}, "
                        f"tail file={xml_line}")

    found_end_of_response = False
    end_of_response = '</RESPONSE>'
    found_incident = False
    incident_text = 'incident signature'
    xml_line = ''.join(xml_lines.replace('\n', '').replace('|', ''))
    if end_of_response in xml_line:
        found_end_of_response = True
    if incident_text in str(xml_line).lower():
        found_incident = True

    if found_incident is True:
        raise Exception(f"WARNING: INCIDENT FOUND, "
                        f"test_by={test_by} "
                        f"directory={directory_name} "
                        f"batch file={batch}, "
                        f"tail file={xml_line}")
    elif found_end_of_response is False:
        raise Exception(f"WARNING: END OF RESPONSE NOT FOUND, "
                        f"test_by={test_by} "
                        f"directory={directory_name} "
                        f"batch file={batch}, "
                        f"tail file={xml_line}")
    else:
        etld_lib_functions.logger.info(f"PASS: "
                                       f"test_by={test_by} "
                                       f"directory={directory_name} "
                                       f"batch file={batch}, "
                                       f"tail file={xml_line}")
    return xml_line


def validate_extract_directories_xml_files_downloaded(test_by='characters'):
    xml_dir_list = []
    xml_dir_list.append(Path(etld_lib_config.host_list_extract_dir))
    xml_dir_list.append(Path(etld_lib_config.host_list_detection_extract_dir))
    xml_dir_list.append(Path(etld_lib_config.kb_extract_dir))
    xml_dir_search_glob = '*.xml.gz'

    for xml_dir in xml_dir_list:
        xml_file_list = sorted(Path(xml_dir).glob(xml_dir_search_glob))
        for xml_file_path in xml_file_list:
            try:
                if 'characters' in test_by:
                    lines = validate_xml_is_closed_properly(open_file_method=gzip.open,
                                                            file_path=xml_file_path,
                                                            number_of_lines=8,
                                                            number_of_characters=1000)
                else:
                    lines = validate_xml_is_closed_properly(test_by='lines',
                                                            open_file_method=gzip.open,
                                                            file_path=xml_file_path,
                                                            number_of_lines=8,
                                                            number_of_characters=1000)
            except Exception as e:
                etld_lib_functions.logger.error(f'Exception {e}')


def test_knowledgebase(test_name='knowledgebase'):
    etld_lib_config.kb_last_modified_after = etld_lib_datetime.get_utc_date_minus_days(30)
    etld_lib_functions.logger.info(f"Starting test with kb_last_modified_after: {etld_lib_config.kb_last_modified_after}")
    try:
        knowledgebase_02_workflow_manager.main()
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info(f"Ending   test with kb_last_modified_after: {etld_lib_config.kb_last_modified_after}")


def test_host_list(test_name='host_list'):
    etld_lib_config.host_list_vm_processed_after = etld_lib_datetime.get_utc_date_minus_days(180)
    etld_lib_config.host_list_test_system_flag = True
    etld_lib_config.host_list_test_number_of_files_to_extract = 3
    etld_lib_config.host_list_distribution_csv_flag = True
    etld_lib_config.host_list_payload_option = {'truncation_limit': '25'}
    etld_lib_functions.logger.info(
        f"Starting test with: "
        f"host_list_vm_processed_after={etld_lib_config.host_list_vm_processed_after}, "
        f"host_list_payload_option={etld_lib_config.host_list_payload_option}, " 
        f"host_list_test_number_of_files_to_extract={etld_lib_config.host_list_test_number_of_files_to_extract}")
    try:
        host_list_02_workflow_manager.main()
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info(f"Ending: {etld_lib_config.host_list_vm_processed_after}, "
                                   f"Test {etld_lib_config.host_list_test_number_of_files_to_extract} files")


def test_host_list_detection(test_name='host_list_detection'):
    etld_lib_config.host_list_detection_vm_processed_after = etld_lib_datetime.get_utc_date_minus_days(180)
    etld_lib_config.host_list_vm_processed_after = etld_lib_datetime.get_utc_date_minus_days(180)
    etld_lib_config.host_list_test_system_flag = True
    etld_lib_config.host_list_test_number_of_files_to_extract = 3
    etld_lib_config.host_list_detection_distribution_csv_flag = True
    etld_lib_config.host_list_payload_option = {'truncation_limit': '25'}
    etld_lib_functions.logger.info(
        f"Starting test with: "
        f"host_list_vm_processed_after={etld_lib_config.host_list_vm_processed_after}, "
        f"host_list_payload_option={etld_lib_config.host_list_payload_option}, " 
        f"host_list_test_number_of_files_to_extract={etld_lib_config.host_list_test_number_of_files_to_extract}")
    try:
        host_list_detection_02_workflow_manager.main()
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info(f"Ending: {etld_lib_config.host_list_vm_processed_after}, "
                                   f"Test {etld_lib_config.host_list_test_number_of_files_to_extract} files")


def test_asset_inventory(test_name='asset_inventory'):
    etld_lib_config.asset_inventory_asset_last_updated = etld_lib_datetime.get_utc_date_minus_days(180)
    etld_lib_config.asset_inventory_test_system_flag = True
    etld_lib_config.asset_inventory_test_number_of_files_to_extract = 3
    etld_lib_config.asset_inventory_distribution_csv_flag = True
    etld_lib_functions.logger.info(
        f"Starting test with: "
        f"asset_inventory_asset_last_updated={etld_lib_config.asset_inventory_asset_last_updated}, "
        f"asset_inventory_test_number_of_files_to_extract="
        f"{etld_lib_config.asset_inventory_test_number_of_files_to_extract}")
    try:
        asset_inventory_02_workflow_manager.main()
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info(
        f"Ending test with: "
        f"asset_inventory_asset_last_updated={etld_lib_config.asset_inventory_asset_last_updated}, "
        f"asset_inventory_test_number_of_files_to_extract="
        f"{etld_lib_config.asset_inventory_test_number_of_files_to_extract}")


def test_was(test_name='was'):
    etld_lib_config.was_webapp_last_scan_date = etld_lib_datetime.get_utc_date_minus_days(365)
    etld_lib_config.was_test_system_flag = True
    etld_lib_config.was_test_number_of_files_to_extract = 3
    etld_lib_config.was_distribution_csv_flag = True
    etld_lib_functions.logger.info(
        f"Starting test with: "
        f"was_webapp_last_scan_date={etld_lib_config.was_webapp_last_scan_date}, "
        f"was_test_number_of_files_to_extract="
        f"{etld_lib_config.was_test_number_of_files_to_extract}")
    try:
        was_02_workflow_manager.main()
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info(
        f"Ending test with: "
        f"was_webapp_last_scan_date={etld_lib_config.was_webapp_last_scan_date}, "
        f"was_test_number_of_files_to_extract="
        f"{etld_lib_config.was_test_number_of_files_to_extract}")


def test_was_count():
    etld_lib_config.remove_old_files(
        dir_path=etld_lib_config.was_extract_dir,
        dir_search_glob=etld_lib_config.was_extract_dir_file_search_blob_webapp_count
    )

    file_info_dict = \
        etld_lib_config.prepare_extract_batch_file_name(
            next_batch_number_str='batch_000001',
            next_batch_date=etld_lib_datetime.get_utc_datetime_qualys_format(),
            extract_dir=etld_lib_config.was_extract_dir,
            file_name_type="was_count_webapp",
            file_name_option="last_scan_date",
            file_name_option_date='1970-01-01T00:00:00Z',
            file_extension="json",
            compression_method=etld_lib_config.was_open_file_compression_method)

    etld_lib_functions.logger.info("Starting test was_04_extract_from_qualys.was_webapp_extract_count")
    try:
        was_04_extract_from_qualys.was_webapp_extract_count(
            batch_number_str='batch_000001',
            qualys_headers_dict={},
            file_info_dict=file_info_dict,
            cred_dict=etld_lib_authentication_objects.qualys_authentication_obj.get_credentials_dict(),
        )
    except Exception as e:
        etld_lib_functions.logger.error(f"Failed Test: Exception{e}")
        raise Exception

    etld_lib_functions.logger.info("Ending test was_04_extract_from_qualys.was_webapp_extract_count")


def main(test_function_to_run='All'):
    etld_lib_config.test_system_do_not_test_intermediary_extracts_flag = True
    return_code_dict = {}
    return_code = 0
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'test_knowledgebase':
            test_knowledgebase()
    except Exception as e:
        return_code_dict['test_knowledgebase'] = 1
        return_code_dict['test_knowledgebase_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'test_host_list':
            test_host_list()
    except Exception as e:
        return_code_dict['test_host_list'] = 1
        return_code_dict['test_host_list_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'test_host_list_detection':
            test_host_list_detection()
    except Exception as e:
        return_code_dict['test_host_list_detection'] = 1
        return_code_dict['test_host_list_detection_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'test_asset_inventory':
            test_asset_inventory()
    except Exception as e:
        return_code_dict['test_asset_inventory'] = 1
        return_code_dict['test_asset_inventory_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'test_was':
            test_was()
    except Exception as e:
        return_code_dict['test_was'] = 1
        return_code_dict['test_was_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'validate_extract_directories_json_files_downloaded':
            validate_extract_directories_json_files_downloaded()
    except Exception as e:
        return_code_dict['test_extract_directories_json_files_downloaded'] = 1
        return_code_dict['test_extract_directories_json_files_downloaded_exception'] = f"{e}"
        return_code = 1
    try:
        if test_function_to_run == 'All' or test_function_to_run == 'validate_extract_directories_xml_files_downloaded':
            validate_extract_directories_xml_files_downloaded(test_by='characters')
    except Exception as e:
        return_code_dict['test_extract_directories_xml_files_downloaded'] = 1
        return_code_dict['test_extract_directories_xml_files_downloaded_exception'] = f"{e}"
        return_code = 1
    # try:
    #     if test_function_to_run == 'test_was_count':
    #         test_was_count()
    # except Exception as e:
    #     return_code_dict['test_was_count'] = 1
    #     return_code_dict['test_was_count_exception'] = f"{e}"
    #     return_code = 1

    for return_code_key in return_code_dict.keys():
        etld_lib_functions.logger(f"TEST FAILED: {return_code_key}={return_code_dict[return_code_key]}")

    exit(return_code)


if __name__ == "__main__":
    etld_lib_functions.main(my_logger_prog_name='etld_lib_test')
    etld_lib_config.main()
    etld_lib_credentials.main()
    etld_lib_authentication_objects.main()
    arguments = sys.argv
    if len(arguments) > 1:
        test_function_to_run = arguments[1]
    else:
        test_function_to_run = 'All'

    main(test_function_to_run=test_function_to_run)



#!/usr/bin/env python3
from qualys_etl.etld_lib import etld_lib_config
from qualys_etl.etld_lib import etld_lib_spawn_etl
from qualys_etl.etld_was import was_02_workflow_manager


# def main():
#     prefix_name = "was_"
#     etld_lib_config.set_path_qetl_user_home_dir()
#     etld_lib_spawn_etl.log_dir = etld_lib_config.qetl_user_log_dir
#     etld_lib_spawn_etl.etl_dir_to_monitor = etld_lib_config.qetl_user_data_dir
#     etld_lib_spawn_etl.log_file_path = getattr(etld_lib_config, f"{prefix_name}log_file")
#     etld_lib_spawn_etl.log_file_rotate_path = getattr(etld_lib_config, f"{prefix_name}log_rotate_file")
#     etld_lib_spawn_etl.lock_file = getattr(etld_lib_config, f"{prefix_name}lock_file")
#     etld_lib_spawn_etl.log_file_max_size = getattr(etld_lib_config, f"{prefix_name}log_file_max_size")                          # 1024 * size = Max Meg Size
#     etld_lib_spawn_etl.spawned_process_max_run_time = getattr(etld_lib_config, f"{prefix_name}spawned_process_max_run_time")    # seconds before terminate as there is an issue.
#     etld_lib_spawn_etl.spawned_process_sleep_time = getattr(etld_lib_config, f"{prefix_name}spawned_process_sleep_time")        # check every n seconds for spawned_process.is_alive()
#     etld_lib_spawn_etl.spawned_process_count_to_status_update = getattr(etld_lib_config, f"{prefix_name}spawned_process_count_to_status_update")  # print status spawned_process.is_alive() after n checks
#     etld_lib_spawn_etl.etl_files_max_time_between_changes = getattr(etld_lib_config, f"{prefix_name}etl_files_max_time_between_changes")
#     etld_lib_spawn_etl.target_module_to_run = getattr(f"{prefix_name}02_workflow_manager", f"{prefix_name}etl_workflow")
#     etld_lib_spawn_etl.target_method_in_module_to_run = f"{prefix_name}etl_workflow"
#     etld_lib_spawn_etl.etl_main()

def main():
    prefix_name = "was"
    etld_lib_spawn_etl.etl_main(etl_workflow_prefix=prefix_name)

if __name__ == '__main__':
    main()
